let VERSION_TIMESTAMP = "1585928184";
let forceCacheUrls = [
	"./",
	"./css.css",
	"./js.js",
	"./manifest.json",
	"./libs/basis/basis_transcoder.js",
	"./libs/basis/basis_transcoder.wasm",
	"./libs/draco/draco_decoder.js",
	"./favicon.ico",
	"./img/icon256.png",
	"./img/icon512.png",

	"./img/buttons/music/default.png",
	"./img/buttons/music/toggled.png",
	"./img/buttons/sfx/default.png",
	"./img/buttons/sfx/toggled.png",
	"./img/buttons/pause/default.png",
	"./img/buttons/pause/active.png",
	"./img/buttons/play/default.png",
	"./img/buttons/play/active.png",
	"./img/buttons/qualitySetting/default.png",
	"./img/buttons/qualitySetting/active.png",
	"./img/buttons/saveMe/default.png",
	"./img/buttons/saveMe/active.png",

	"./CubicFive01.woff2",

	"./img/doubleDodgersLogo.png",
	"./img/appleAppStore.png",
	"./img/googlePlayStore.png",

	"./img/arrowShareHats.png",
	"./img/facebookShare.png",
	"./img/twitterShare.png",

	"./img/shareButtons/Discord.png",
	"./img/shareButtons/Twitter.png",
	"./img/shareButtons/Instagram.png",
	"./img/shareButtons/Youtube.png",
];

let cacheFallbackUrls = [
	"/highScores",
	"/myProfile",
];

self.addEventListener("install", e => {
	e.waitUntil(onInstall());
});

async function onInstall(){
	await self.skipWaiting();
	if(VERSION_TIMESTAMP != "000"){
		let cache = await openCache();
		await cache.addAll(forceCacheUrls.map(url => new Request(url+"?v="+VERSION_TIMESTAMP)));
	}
}

self.addEventListener("activate", e => {
	e.waitUntil(onActivate());
});

async function onActivate(){
	//clear old caches
	let keys = await caches.keys();
	for(const key of keys){
		let result = /mainCacheV(\d+)/.exec(key);
		if(result && result[1] != VERSION_TIMESTAMP){
			await caches.delete(key);
		}
	}
}

self.addEventListener("fetch", e => {
	if(VERSION_TIMESTAMP === "000") return;
	let url = new URL(e.request.url);
	let origin = url.origin;
	if(url.href.startsWith(self.registration.scope) || url.hostname == "doubledodgers.com"){
		if(url.pathname == "/ytChannel"){
			e.respondWith(getUrlFromProfilePictureCache(e));
		}else if(cacheFallbackUrls.includes(url.pathname)){
			e.respondWith(getUrlFromFallbackCache(e));
		}else{
			e.respondWith(getUrlFromCache(e));
		}
	}
});

async function getUrlFromCache(e){
	let cacheResponse = await caches.match(e.request, {
		ignoreSearch: true,
	});
	if(cacheResponse) return cacheResponse;

	let fetchResponse = await fetch(e.request);
	if((fetchResponse.ok || fetchResponse.type == "opaque") && e.request.method == "GET"){
		let cache = await openCache();
		cache.put(e.request, fetchResponse.clone())
	}
	return fetchResponse;
}

async function getUrlFromFallbackCache(e){
	let fetchResponse = null;
	try{
		fetchResponse = await fetch(e.request);
	}catch(_){}
	if(fetchResponse && fetchResponse.ok){
		let cache = await openCache();
		cache.put(e.request, fetchResponse.clone());
		return fetchResponse;
	}else{
		let cacheResponse = await caches.match(e.request, {
			ignoreSearch: true,
		});
		if(cacheResponse) return cacheResponse;
	}
	return fetchResponse;
}

async function getUrlFromProfilePictureCache(e){
	const expireHeaderKey = "X-DD-fetchTime";
	let cache = await caches.open("profilePictures");
	let cacheResponse = await cache.match(e.request);
	if(cacheResponse){
		let expireValue = cacheResponse.headers.get("X-DD-fetchTime");
		let expireInt = parseInt(expireValue);
		if(!isNaN(expireInt) && Date.now() < expireInt + 1000*60*60*24*10){ //cache for 10 days
			return cacheResponse;
		}
	}
	let fetchResponse = await fetch(e.request);
	let clonedFetchResponse = new Response(fetchResponse.body, fetchResponse);
	clonedFetchResponse.headers.set(expireHeaderKey, Date.now());
	cache.put(e.request, clonedFetchResponse.clone());
	return clonedFetchResponse;
}

async function openCache(){
	return await caches.open("mainCacheV"+VERSION_TIMESTAMP);
}
